using System;
using System.Collections;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Security;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading;
using System.Windows.Input;
using System.Xml;
using WpfCoreExceptionDialog.Dialogs.Resources;

namespace WpfCoreExceptionDialog.Dialogs
{
    /// <summary>
    /// Stellt die Funktionalitt fr die Behandlung von Ausnahmen zur Verfgung.
    /// </summary>
    public class ExceptionParser
    {
        /// <summary>
        /// Ruft das Containerobjekt fr die Verarbeitung der Ausnahmen ab.
        /// </summary>
        public ExceptionPackage CurrentPackage { get; private set; }

        /// <summary>
        /// Ldt aus einer Ausnahme die verfgbaren Werte.
        /// </summary>
        /// <param name="exception">Die Ausnahme, die geladen wird.</param>
        public void Load(Exception exception)
        {
            Mouse.OverrideCursor = Cursors.Wait;
            ExceptionPackage ep = new ExceptionPackage()
            {
                General = new GeneralPoint(),
                Assemblies = new AssemblyPointCollection()
            };

            #region --[ General ]--

            ep.General.Title = GetAssemblyProduct(Assembly.GetEntryAssembly());
            ep.General.Version = GetAssemblyVersion(Assembly.GetEntryAssembly());
            ep.General.StartupPath = Path.GetDirectoryName(Assembly.GetExecutingAssembly().Location);
            ep.General.Operatingsystem = GetOSVersion;
            ep.General.NetVersion = GetFrameworkVersion;
            ep.General.Host = GetHostName;
            ep.General.User = GetWindowsIdentity;
            ep.General.Assembly = GetAssemblyName();
            ep.General.AppDomain = AppDomain.CurrentDomain.FriendlyName;
            ep.General.ThreadName = GetThreadName;
            ep.General.ThreadUser = Environment.UserName;
            ep.General.ThreadId = Thread.CurrentThread.ManagedThreadId.ToString();

            // Rekursiv durchlaufen
            Exception _currentException = exception;
            ExceptionPoint _currentExceptionPoint;
            ExceptionPoint _beforeExceptionPoint = null;

            do
            {
                _currentExceptionPoint = new ExceptionPoint()
                {
                    Stacks = new StackPointCollection(),
                    DataProperties = new DataPropertyCollection()
                };

                if (_currentException.Message != null && _currentException.Message.Length > 0)
                    _currentExceptionPoint.Message = _currentException.Message;
                else
                    _currentExceptionPoint.Message = SR.DEFAULTEXCEPTIONMESSAGE;

                if (_currentException.HelpLink == null ||
                    _currentException.HelpLink.Trim().Length == 0)
                {
                    _currentExceptionPoint.HelpLink = "";
                }
                else
                {
                    _currentExceptionPoint.HelpLink = _currentException.HelpLink;
                }

                _currentExceptionPoint.Type = GetExceptionType(_currentException);
                _currentExceptionPoint.Method = GetMethodName(_currentException);
                _currentExceptionPoint.Line = GetFileLineNumber(_currentException).ToString();
                _currentExceptionPoint.Column = GetFileColumnNumber(_currentException).ToString();
                _currentExceptionPoint.Sourcefile = GetFileName(_currentException);
                _currentExceptionPoint.Class = GetClassName(_currentException);
                _currentExceptionPoint.IlOffset = GetILOffset(_currentException).ToString();
                _currentExceptionPoint.NativeOffset = GetNativeOffset(_currentException).ToString();

                // anzuzeigende erweiterte Eigenschaften
                _currentExceptionPoint.DataProperties = GetDataProperties(_currentException);

                // Stackseintrge

                _currentExceptionPoint.Stacks.Clear();

                string[] stacks = GetStackTrace(_currentException);
                if (stacks != null && stacks.Length > 0)
                {
                    foreach (string stack in stacks)
                    {
                        StackPoint sp = new StackPoint();
                        sp.Text = stack.Trim();
                        _currentExceptionPoint.Stacks.Add(sp);
                    }
                }

                if (_beforeExceptionPoint == null)
                {
                    // 1.
                    ep.General.Exception = _currentExceptionPoint;
                    _beforeExceptionPoint = _currentExceptionPoint;
                }
                else
                {
                    // andere
                    _beforeExceptionPoint.InnerException = _currentExceptionPoint;
                    _beforeExceptionPoint = _currentExceptionPoint;
                }

                // nchste Exception
                if (_currentException.InnerException == null)
                {
                    _currentException = null;
                }
                else
                {
                    _currentException = _currentException.InnerException;
                }
            } while (_currentException != null);

            #endregion --[ General ]--

            #region --[ Assemblies ]--

            // alle Assemblies durchgehen

            ep.Assemblies.Clear();

            AssemblyPoint[] assemblyPoints = GetLoadedAssemblies();
            foreach (AssemblyPoint assemblyPoint in assemblyPoints)
            {
                // TODO: Exception auswerten
                ep.Assemblies.Add(assemblyPoint);
            }

            #endregion --[ Assemblies ]--

            //this.CurrentException = ep.General.Exception;
            this.CurrentPackage = ep;
            Mouse.OverrideCursor = null;
        }

        /// <summary>
        /// Serialisiert die aktuelle Ausnahme.
        /// </summary>
        /// <returns>Ein <see cref="XmlDocument"/>, das die serialisierte Ausnamme reprsentiert.</returns>
        public XmlDocument GetAsXml()
        {
            System.Xml.Serialization.XmlSerializer serializer = new System.Xml.Serialization.XmlSerializer(typeof(ExceptionPackage));
            StringWriter writer = new StringWriter();
            serializer.Serialize(writer, this.CurrentPackage);
            string xml = writer.ToString();
            writer.Close();
            serializer = null;

            XmlDocument xmlDoc = new XmlDocument();
            xmlDoc.LoadXml(xml);
            return xmlDoc;
        }

        /// <summary>
        /// Serialisiert die aktuelle Ausnahme zu einem Html-Text.
        /// </summary>
        /// <returns>Ein String, das die serialisierte Ausnamme reprsentiert.</returns>
        public string GetAsHtmlString()
        {
            try
            {
                string rowTemplate = "<tr><td style='white-space: nowrap;width:10px;vertical-align: top; padding:2px; padding-top:0px;padding-left:0px'>{0}:</td><td style='width:100%;padding:2px;padding-top:0px;padding-right:0px'>{1}</td></tr>";
                string spacerTemplate = "<tr><td><hr style='border: 0;height: 1px;color:silver'></td><td><hr style='border: 0;height: 1px;color:silver'></td></tr>";
                StringBuilder tb = new StringBuilder();

                tb.Append("<table width='100%' cellspacing='0' cellpadding='0' style='font-size: 10pt; font-family:Arial; border: 0'>\n");

                ExceptionPoint currentEp = this.CurrentPackage.General.Exception;
                while (currentEp != null)
                {
                    tb.AppendFormat(
                        rowTemplate,
                        SR.GENERALTABMESSSAGE,
                        NormalizeToHtml(String.IsNullOrEmpty(currentEp.Message) ? "-" : currentEp.Message)
                    );

                    // erweiterte Eigenschaften
                    foreach (DataProperty dp in currentEp.DataProperties)
                    {
                        tb.AppendFormat(
                            rowTemplate,
                            NormalizeToHtml(dp.Name),
                            NormalizeToHtml(dp.Value)
                        );
                    }

                    tb.AppendFormat(
                        rowTemplate,
                        SR.GENERALTABTYPE,
                        NormalizeToHtml(String.IsNullOrEmpty(currentEp.Type) ? "-" : currentEp.Type)
                    );

                    tb.AppendFormat(
                        rowTemplate,
                        SR.GENERALTABSOURCEFILE,
                        NormalizeToHtml(String.IsNullOrEmpty(currentEp.Sourcefile) ? "-" : currentEp.Sourcefile)
                    );

                    tb.AppendFormat(
                        rowTemplate,
                        SR.GENERALTABCLASS,
                        NormalizeToHtml(String.IsNullOrEmpty(currentEp.Class) ? "-" : currentEp.Class)
                    );

                    tb.AppendFormat(
                        rowTemplate,
                        SR.GENERALTABMETHOD,
                        NormalizeToHtml(String.IsNullOrEmpty(currentEp.Method) ? "-" : currentEp.Method)
                    );

                    tb.AppendFormat(
                        rowTemplate,
                        SR.GENERALTABLINECOLUMN,
                        NormalizeToHtml((String.IsNullOrEmpty(currentEp.Line) ? "-" : currentEp.Line) + "/" + (String.IsNullOrEmpty(currentEp.Column) ? "-" : currentEp.Column))
                    );

                    // untergeordnete holen
                    if (currentEp.InnerException != null)
                    {
                        currentEp = currentEp.InnerException;
                        // Space
                        tb.AppendFormat(spacerTemplate);
                    }
                    else
                    {
                        currentEp = null;
                    }
                }

                tb.Append("</table>\n<br/>\n<br/>");

                return "<div>" + tb.ToString() + "</div>";
            }
            catch
            {
                return "";
            }
        }

        /// <summary>
        /// Serialisiert die aktuelle Ausnahme zu Text.
        /// </summary>
        /// <returns>Ein <see cref="String"/>, das die serialisierte Ausnamme reprsentiert.</returns>
        /// <param name="onlyExceptionTree">Erzeugt einen verkrzt Typ.</param>
        public string GetAsText(bool onlyExceptionTree = false)
        {
            StringBuilder sb = new StringBuilder();

            if (onlyExceptionTree == false)
            {
                // Allgemein
                // -------------------------------------------
                sb.Append("==========================\n");
                sb.AppendFormat
                    (
                        "{0}\n",
                        SR.GENERALTABTEXT.Replace(":", "").Replace("&", "")
                    );
                sb.Append("==========================\n");
                sb.AppendFormat
                    (
                        "  {0}: {1}\n",
                        SR.APPLICATIONTABTITLE,
                        String.IsNullOrEmpty(this.CurrentPackage.General.Title) ? "-" : this.CurrentPackage.General.Title
                    );
                sb.AppendFormat
                    (
                        "  {0}: {1}\n",
                        SR.APPLICATIONTABVERSION,
                        String.IsNullOrEmpty(this.CurrentPackage.General.Version) ? "-" : this.CurrentPackage.General.Version
                    );
                sb.AppendFormat
                    (
                        "  {0}: {1}\n",
                        SR.APPLICATIONTABSTARTUPPATH,
                        String.IsNullOrEmpty(this.CurrentPackage.General.StartupPath) ? "-" : this.CurrentPackage.General.StartupPath
                    );
                sb.AppendFormat
                    (
                        "  {0}: {1}\n",
                        SR.APPLICATIONTABOS,
                        String.IsNullOrEmpty(this.CurrentPackage.General.Operatingsystem) ? "-" : this.CurrentPackage.General.Operatingsystem
                    );
                sb.AppendFormat
                    (
                        "  {0}: {1}\n",
                        SR.APPLICATIONTABNETVERSION,
                        String.IsNullOrEmpty(this.CurrentPackage.General.NetVersion) ? "-" : this.CurrentPackage.General.NetVersion
                    );
                sb.AppendFormat
                    (
                        "  {0}: {1}\n",
                        SR.APPLICATIONTABHOST,
                        String.IsNullOrEmpty(this.CurrentPackage.General.Host) ? "-" : this.CurrentPackage.General.Host
                    );
                sb.AppendFormat
                    (
                        "  {0}: {1}\n",
                        SR.APPLICATIONTABUSERNAME,
                        String.IsNullOrEmpty(this.CurrentPackage.General.User) ? "-" : this.CurrentPackage.General.User
                    );
                sb.AppendFormat
                    (
                        "  {0}: {1}\n",
                        SR.ASSEMBLIESTABASSEMBLYTEXT,
                        String.IsNullOrEmpty(this.CurrentPackage.General.Assembly) ? "-" : this.CurrentPackage.General.Assembly
                    );
                sb.AppendFormat
                    (
                        "  {0}: {1}\n",
                        SR.APPLICATIONTABAPPDOMAIN,
                        String.IsNullOrEmpty(this.CurrentPackage.General.AppDomain) ? "-" : this.CurrentPackage.General.AppDomain
                    );
                sb.AppendFormat
                    (
                        "  {0}: {1}/{2}/{3}\n",
                        SR.APPLICATIONTABTHREADNAMEUSERID,
                        String.IsNullOrEmpty(this.CurrentPackage.General.ThreadName) ? "-" : this.CurrentPackage.General.ThreadName,
                        String.IsNullOrEmpty(this.CurrentPackage.General.ThreadUser) ? "-" : this.CurrentPackage.General.ThreadUser,
                        String.IsNullOrEmpty(this.CurrentPackage.General.ThreadId) ? "-" : this.CurrentPackage.General.ThreadId
                    );

                // Ausnahmen
                // -------------------------------------------
                sb.Append("\n");
                sb.Append("==========================\n");
                sb.AppendFormat
                    (
                        "{0}\n",
                        SR.GENERALTABEXCEPTIONS.Replace("_", "").Replace(":", "").Replace("&", "")
                    );
                sb.Append("==========================\n");
            }

            ExceptionPoint currentEp = this.CurrentPackage.General.Exception;

            // RootException
            GetExceptionPointAsText
                (
                    sb,
                    currentEp,
                    false,
                    SR.GENERALTABSOURCEFILE,
                    SR.GENERALTABCLASS,
                    SR.GENERALTABMETHOD,
                    SR.GENERALTABLINECOLUMN,
                    SR.GENERALTABSTACK.Replace("_", "").Replace(":", "") + ":",
                    SR.GENERALTABMESSSAGE,
                    SR.GENERALTABTYPE,
                    onlyExceptionTree
                );

            // alle untergeordneten
            while (currentEp.InnerException != null)
            {
                GetExceptionPointAsText
                    (
                        sb,
                        currentEp.InnerException,
                        true,
                        SR.GENERALTABSOURCEFILE,
                        SR.GENERALTABCLASS,
                        SR.GENERALTABMETHOD,
                        SR.GENERALTABLINECOLUMN,
                        SR.GENERALTABSTACK.Replace("_", "").Replace(":", "") + ":",
                        SR.GENERALTABMESSSAGE,
                        SR.GENERALTABTYPE,
                        onlyExceptionTree
                    );
                currentEp = currentEp.InnerException;
            }

            if (onlyExceptionTree == false)
            {
                // Geladene Dateien
                // -------------------------------------------
                sb.Append("\n");
                sb.Append("==========================\n");
                sb.AppendFormat
                    (
                        "{0}\n",
                        SR.ASSEMBLIESTABTEXT.Replace(":", "").Replace("&", "")
                    );
                sb.Append("==========================\n");
                bool isFirst = true;
                foreach (AssemblyPoint ap in this.CurrentPackage.Assemblies)
                {
                    // beim ersten keine Leerzeile
                    if (isFirst == false)
                        sb.Append("\n");

                    sb.AppendFormat
                        (
                            "> {0}: {1}\n",
                            SR.ASSEMBLIESTABASSEMBLYTEXT,
                            String.IsNullOrEmpty(ap.Name) ? "-" : ap.Name
                        );
                    sb.AppendFormat
                        (
                            "  {0}: {1}\n",
                            SR.ASSEMBLIESTABFULLNAMETEXT,
                            String.IsNullOrEmpty(ap.FullName) ? "-" : ap.FullName
                        );
                    sb.AppendFormat
                        (
                            "  {0}: {1}\n",
                            SR.ASSEMBLIESTABCODEBASETEXT,
                            String.IsNullOrEmpty(ap.Codebase) ? "-" : ap.Codebase
                        );
                    sb.AppendFormat
                        (
                            "  {0}: {1}\n",
                            SR.ASSEMBLIESTABLOCATIONTEXT,
                            String.IsNullOrEmpty(ap.Location) ? "-" : ap.Location
                        );
                    sb.AppendFormat
                        (
                            "  {0}: {1}\n",
                            SR.ASSEMBLIESTABIMAGEVERSIONTEXT,
                            String.IsNullOrEmpty(ap.Imageversion) ? "-" : ap.Imageversion
                        );
                    sb.AppendFormat
                        (
                            "  {0}: {1}\n",
                            SR.ASSEMBLIESTABLOADFROMCACHETEXT,
                            String.IsNullOrEmpty(ap.LoadFromcache.GetString()) ? "-" : ap.LoadFromcache.GetString()
                        );

                    if (ap.Attributes != null)
                    {
                        sb.AppendFormat
                            (
                                "  {0}\n",
                                SR.ASSEMBLIESTABFILEATTRIBUTETEXT + ":"
                            );
                        sb.AppendFormat
                            (
                                "      {0}: {1}\n",
                                SR.ASSEMBLIESTABHIDDENTEXT,
                                String.IsNullOrEmpty(ap.Attributes.Hidden.GetString()) ? "-" : ap.Attributes.Hidden.GetString()
                            );
                        sb.AppendFormat
                            (
                                "      {0}: {1}\n",
                                SR.ASSEMBLIESTABREADONLYTEXT,
                                String.IsNullOrEmpty(ap.Attributes.ReadOnly.GetString()) ? "-" : ap.Attributes.ReadOnly.GetString()
                            );
                        sb.AppendFormat
                            (
                                "      {0}: {1}\n",
                                SR.ASSEMBLIESTABARCHIVETEXT,
                                String.IsNullOrEmpty(ap.Attributes.Archive.GetString()) ? "-" : ap.Attributes.Archive.GetString()
                            );
                        sb.AppendFormat
                            (
                                "      {0}: {1}\n",
                                SR.ASSEMBLIESTABCOMPRESSTEXT,
                                String.IsNullOrEmpty(ap.Attributes.Compress.GetString()) ? "-" : ap.Attributes.Compress.GetString()
                            );
                        sb.AppendFormat
                            (
                                "      {0}: {1}\n",
                                SR.ASSEMBLIESTABENCRYPTTEXT,
                                String.IsNullOrEmpty(ap.Attributes.Encrypted.GetString()) ? "-" : ap.Attributes.Encrypted.GetString()
                            );
                    }
                    isFirst = false;
                }
            }

            // Rckgabe
            string temp = sb.ToString();
            sb = null;
            return temp;
        }

        /// <summary>
        /// Serialisiert einen ExceptionPoint mit den angegebenen Parametern in einen StringBuilder.
        /// </summary>
        private void GetExceptionPointAsText(StringBuilder sb,
            ExceptionPoint currentEp,
            bool beforeLine,
            string generalTabSourceFile,
            string generalTabClass,
            string generalTabMethod,
            string generalTabLineColumn,
            string generalTabStack,
            string meldung,
            string typ,
            bool small)
        {
            // leere Zeile einfgen
            if (beforeLine == true)
                sb.Append("\n");

            sb.AppendFormat
                (
                    "* {0}: {1}\n",
                    meldung,
                    String.IsNullOrEmpty(currentEp.Message) ? "-" : currentEp.Message
                );
            // erweiterte Eigenschaften
            foreach (DataProperty dp in currentEp.DataProperties)
            {
                string value = dp.Value.Trim();
                if (value.StartsWith("{") &&
                    value.EndsWith("}"))
                {
                    // json
                    value = "\n" + value;
                }
                if (value.StartsWith("[") &&
                    value.EndsWith("]"))
                {
                    // json
                    value = "\n" + value;
                }

                sb.AppendFormat
                    (
                        "  {0}: {1}\n",
                        dp.Name,
                        value
                    );
            }
            sb.AppendFormat
                (
                    "  {0}: {1}\n",
                    typ,
                    String.IsNullOrEmpty(currentEp.Type) ? "-" : currentEp.Type
                );
            sb.AppendFormat
                (
                    "  {0}: {1}\n",
                    generalTabSourceFile,
                    String.IsNullOrEmpty(currentEp.Sourcefile) ? "-" : currentEp.Sourcefile
                );
            sb.AppendFormat
                (
                    "  {0}: {1}\n",
                    generalTabClass,
                    String.IsNullOrEmpty(currentEp.Class) ? "-" : currentEp.Class
                );
            sb.AppendFormat
                (
                    "  {0}: {1}\n",
                    generalTabMethod,
                    String.IsNullOrEmpty(currentEp.Method) ? "-" : currentEp.Method
                );
            sb.AppendFormat
                (
                    "  {0}: {1}/{2}\n",
                    generalTabLineColumn,
                    String.IsNullOrEmpty(currentEp.Line) ? "-" : currentEp.Line,
                    String.IsNullOrEmpty(currentEp.Column) ? "-" : currentEp.Column
                );

            if (small == false)
            {
                // Stack schreiben
                string stackTitle = Guid.NewGuid().ToString();
                sb.AppendFormat("  {0}", stackTitle);
                bool hasStack = false;
                foreach (StackPoint sp in currentEp.Stacks)
                {
                    if (sp.Text.Trim().Length > 0)
                    {
                        hasStack = true;
                        sb.AppendFormat("    * {0}\n", sp.Text);
                    }
                }

                if (hasStack == true)
                    sb = sb.Replace(stackTitle, generalTabStack.Replace("&", "") + "\n");
                else
                    sb = sb.Replace(stackTitle, generalTabStack.Replace("&", "") + " -\n");
            }
        }

        /// <summary>
        /// Ermittelt den Dateinamen der ausfhrenden Assembly ohne Verzeichnis.
        /// </summary>
        private string GetAssemblyName()
        {
            string assemblyName = "";

            try
            {
                assemblyName = System.Reflection.AssemblyName.GetAssemblyName(Assembly.GetExecutingAssembly().Location).ToString();
                assemblyName = assemblyName.Substring(0, assemblyName.IndexOf(",", assemblyName.IndexOf(",") + 1));
            }
            catch
            {
                assemblyName = "";
            }

            return assemblyName;
        }

        /// <summary>
        /// Ermittelt den aktuellen Namen des Hosts.
        /// </summary>
        private string GetHostName
        {
            get
            {
                try
                {
                    string serverName = System.Environment.MachineName;
                    return System.Net.Dns.GetHostEntry(serverName).HostName;
                }
                catch (SecurityException)
                {
                    return SR.DEFAULTPERMISSIONDENIEDMESSAGE;
                }
                catch
                {
                    return SR.DEFAULTGENERALDENIEDMESSAGE;
                }
            }
        }

        /// <summary>
        /// Ermittelt das aktuelle Datum.
        /// </summary>
        private string GetTimeStamp
        {
            get
            {
                try
                {
                    return DateTime.Now.ToString();
                }
                catch (SecurityException)
                {
                    return SR.DEFAULTPERMISSIONDENIEDMESSAGE;
                }
                catch
                {
                    return SR.DEFAULTGENERALDENIEDMESSAGE;
                }
            }
        }

        /// <summary>
        /// Ermittelt den voll qualifizieren Namen der ausfhrenden Assembly.
        /// </summary>
        private string GetAssemblyFullName
        {
            get
            {
                try
                {
                    return Assembly.GetExecutingAssembly().FullName;
                }
                catch (SecurityException)
                {
                    return SR.DEFAULTPERMISSIONDENIEDMESSAGE;
                }
                catch
                {
                    return SR.DEFAULTGENERALDENIEDMESSAGE;
                }
            }
        }

        /// <summary>
        /// Ermittelt den Pfad oder UNC-Pfad der Datei, die das Manifest enthlt.
        /// </summary>
        private string GetAssemblyLocation
        {
            get
            {
                try
                {
                    return Assembly.GetExecutingAssembly().Location;
                }
                catch (SecurityException)
                {
                    return SR.DEFAULTPERMISSIONDENIEDMESSAGE;
                }
                catch
                {
                    return SR.DEFAULTGENERALDENIEDMESSAGE;
                }
            }
        }

        /// <summary>
        /// Ermittelt den Namen der AppDomain fr die aktuelle Anforderung.
        /// </summary>
        private string GetAppDomainName
        {
            get
            {
                try
                {
                    return AppDomain.CurrentDomain.FriendlyName;
                }
                catch (SecurityException)
                {
                    return SR.DEFAULTPERMISSIONDENIEDMESSAGE;
                }
                catch
                {
                    return SR.DEFAULTGENERALDENIEDMESSAGE;
                }
            }
        }

        /// <summary>
        /// Ermittelt den Namen der Identitt des Threads fr die aktuelle Anforderung.
        /// </summary>
        private string GetThreadIdentity
        {
            get
            {
                try
                {
                    return Thread.CurrentPrincipal.Identity.Name;
                }
                catch (SecurityException)
                {
                    return SR.DEFAULTPERMISSIONDENIEDMESSAGE;
                }
                catch
                {
                    return SR.DEFAULTGENERALDENIEDMESSAGE;
                }
            }
        }

        /// <summary>
        /// Ermittelt die Id des Threads fr die aktuelle Anforderung.
        /// </summary>
        private string GetThreadId
        {
            get
            {
                try
                {
                    return Thread.CurrentThread.ManagedThreadId.ToString();
                }
                catch (SecurityException)
                {
                    return SR.DEFAULTPERMISSIONDENIEDMESSAGE;
                }
                catch
                {
                    return SR.DEFAULTGENERALDENIEDMESSAGE;
                }
            }
        }

        /// <summary>
        /// Ermittelt den Namen des Threads fr die aktuelle Anforderung.
        /// </summary>
        private string GetThreadName
        {
            get
            {
                try
                {
                    string name = Thread.CurrentThread.Name;
                    if (string.IsNullOrEmpty(name))
                        name = "-";

                    return name;
                }
                catch (SecurityException)
                {
                    return SR.DEFAULTPERMISSIONDENIEDMESSAGE;
                }
                catch
                {
                    return SR.DEFAULTGENERALDENIEDMESSAGE;
                }
            }
        }

        /// <summary>
        /// Ermittelt den Namen des Benutzers fr die aktuelle Anforderung.
        /// </summary>
        private string GetWindowsIdentity
        {
            get
            {
                try
                {
                    return System.Security.Principal.WindowsIdentity.GetCurrent().Name;
                }
                catch (SecurityException)
                {
                    return SR.DEFAULTPERMISSIONDENIEDMESSAGE;
                }
                catch
                {
                    return SR.DEFAULTGENERALDENIEDMESSAGE;
                }
            }
        }

        /// <summary>
        /// Ermittelt die Version des Betriebssystems.
        /// </summary>
        private string GetOSVersion
        {
            get
            {
                try
                {
                    return Environment.OSVersion.ToString();
                }
                catch (SecurityException)
                {
                    return SR.DEFAULTPERMISSIONDENIEDMESSAGE;
                }
                catch
                {
                    return SR.DEFAULTGENERALDENIEDMESSAGE;
                }
            }
        }

        /// <summary>
        /// Ermittelt die aktuelle .Net-Frameworkversion.
        /// </summary>
        private string GetFrameworkVersion
        {
            get
            {
                try
                {
                    return Environment.Version.ToString();
                }
                catch (SecurityException)
                {
                    return SR.DEFAULTPERMISSIONDENIEDMESSAGE;
                }
                catch
                {
                    return SR.DEFAULTGENERALDENIEDMESSAGE;
                }
            }
        }

        /// <summary>
        /// Extrahiert die Zeilennummer aus dem StackTrace.
        /// </summary>
        private string GetLineNumberFromStack(string stackTrace)
        {
            string lineNumber = "";
            try
            {
                Regex regLine = new Regex(@"(\:{1}line [1-9]+)");
                Match matchLine = regLine.Match(stackTrace);
                if (matchLine != null)
                    lineNumber = matchLine.Value.Replace(":line", "").Trim();
            }
            catch
            {
            }

            return lineNumber;
        }

        /// <summary>
        /// Extrahiert die Spaltenposition in der Datei aus
        /// dem StackTrace der aktuellen Ausnahme.
        /// </summary>
        private int GetFileColumnNumber(System.Exception ex)
        {
            int column = 0;

            try
            {
                // Erstes StackFrame aus StackTrace ermitteln
                StackFrame frame = GetFirstStackFrame(ex);
                if (frame == null)
                    column = 0;
                else
                    // Dateinamen aus StackFrame ermitteln
                    column = frame.GetFileColumnNumber();
            }
            catch
            {
                column = 0;
            }

            return column;
        }

        /// <summary>
        /// Extrahiert die Zeilenposition in der Datei
        /// aus dem StackTrace der aktuellen Ausnahme.
        /// </summary>
        private int GetFileLineNumber(System.Exception ex)
        {
            int line = 0;

            try
            {
                // Erstes StackFrame aus StackTrace ermitteln
                StackFrame frame = GetFirstStackFrame(ex);

                // Dateinamen aus StackFrame ermitteln
                if (frame != null)
                    line = frame.GetFileLineNumber();
                else
                    line = Convert.ToInt32(GetLineNumberFromStack(ex.StackTrace));
            }
            catch
            {
                line = 0;
            }

            return line;
        }

        /// <summary>
        /// Extrahiert den Dateinamen aus dem StackTrace.
        /// </summary>
        private string GetFileNameFromStack(string stackTrace)
        {
            string fileName = "";
            try
            {
                Regex regFile = new Regex(@"(\\{1}[_a-zA-Z0-9\.]+\:)");
                Match matchFile = regFile.Match(stackTrace);
                if (matchFile != null)
                    fileName = matchFile.Value.Trim().Replace("\\", "").Replace(":", "");
            }
            catch
            {
            }

            return fileName;
        }

        /// <summary>
        /// Extrahiert den Dateinamen aus dem StackTrace der aktuellen Ausnahme.
        /// </summary>
        private string GetFileName(Exception ex)
        {
            string fileName = "";

            try
            {
                // Erstes StackFrame aus StackTrace ermitteln
                StackFrame frame = GetFirstStackFrame(ex);

                // Dateinamen aus StackFrame ermitteln
                if (frame != null)
                {
                    fileName = frame.GetFileName();
                    if (!string.IsNullOrEmpty(fileName))
                        fileName = fileName.Substring(fileName.LastIndexOf("\\") + 1);
                    else
                        fileName = "";
                }
                else
                    fileName = GetFileNameFromStack(ex.StackTrace);
            }
            catch
            {
                fileName = "";
            }

            return fileName;
        }

        /// <summary>
        /// Extrahiert die Methodennamen aus dem StackTrace.
        /// </summary>
        private string GetMethodNameFromStack(string stackTrace)
        {
            string methodName = "";
            try
            {
                Regex regMethod = new Regex(@"\.[_a-zA-Z0-9\, ]+\({1}[_a-zA-Z0-9\, ]+\)");
                Match matchMethod = regMethod.Match(stackTrace);
                if (matchMethod != null)
                    methodName = matchMethod.Value.Replace(".", "").Trim();
            }
            catch
            {
            }

            return methodName;
        }

        /// <summary>
        /// Extrahiert den IlOffset im Code
        /// aus dem StackTrace der aktuellen Ausnahme.
        /// </summary>
        private int GetILOffset(Exception ex)
        {
            int ilOffset = 0;

            try
            {
                // Erstes StackFrame aus StackTrace ermitteln
                StackFrame frame = GetFirstStackFrame(ex);
                if (frame == null)
                    ilOffset = 0;
                else
                    // Dateinamen aus StackFrame ermitteln
                    ilOffset = frame.GetILOffset();
            }
            catch
            {
                ilOffset = 0;
            }

            return ilOffset;
        }

        /// <summary>
        /// Extrahiert den NativeOffset im Code aus
        /// dem StackTrace der aktuellen Ausnahme.
        /// </summary>
        private int GetNativeOffset(Exception ex)
        {
            int nativeOffset = 0;

            try
            {
                // Erstes StackFrame aus StackTrace ermitteln
                StackFrame frame = GetFirstStackFrame(ex);
                if (frame == null)
                    nativeOffset = 0;
                else
                    // Dateinamen aus StackFrame ermitteln
                    nativeOffset = frame.GetNativeOffset();
            }
            catch
            {
                nativeOffset = 0;
            }

            return nativeOffset;
        }

        /// <summary>
        /// Extrahiert den Methodennamen aus dem
        /// StackTrace der aktuellen Ausnahme.
        /// </summary>
        private string GetMethodName(Exception ex)
        {
            string methodName = "";

            try
            {
                // Erstes StackFrame aus StackTrace ermitteln
                StackFrame frame = GetFirstStackFrame(ex);

                // Klassen- und Funktionsnamen aus StackFrame ermitteln
                if (frame != null)
                {
                    MethodBase method = frame.GetMethod();
                    methodName = method.Name + "(";

                    // Methoden-Parameter ermitteln
                    ParameterInfo[] paramInfos = method.GetParameters();
                    string methodParams = "";
                    foreach (ParameterInfo paramInfo in paramInfos)
                    {
                        methodParams += ", " + paramInfo.ParameterType.Name + " " + paramInfo.Name;
                    }
                    if (methodParams.Length > 2)
                        methodName += methodParams.Substring(2);
                    methodName += ")";
                }
                else
                    methodName = GetMethodNameFromStack(ex.StackTrace);
            }
            catch
            {
                methodName = "";
            }

            return methodName;
        }

        /// <summary>
        /// Gibt das StackTrace der aktuellen Ausnahme zurck.
        /// </summary>
        private string[] GetStackTrace(Exception ex)
        {
            try
            {
                if (ex.StackTrace != null)
                {
                    List<string> sp = ex.StackTrace.Split(
                        new[] { "\r\n", "\r", "\n" },
                        StringSplitOptions.None
                    ).ToList();
                    sp = sp.Select(i => i.Trim()).ToList();
                    sp = sp.Where(i => i.StartsWith('-') == false).ToList();
                    return sp.ToArray();
                }
                return new string[0];
            }
            catch
            {
            }

            return new string[0];
        }

        /// <summary>
        /// Extrahiert die Klassennamen aus dem Stack.
        /// </summary>
        private string GetClassNameFromStack(string stackTrace)
        {
            string className = "";

            try
            {
                Regex regClass = new Regex(@" at ([_a-zA-Z0-9\.]+\({1})");
                Match matchClass = regClass.Match(stackTrace);
                if (matchClass != null)
                {
                    className = matchClass.Value.Replace(" at ", "");
                    className = className.Substring(0, className.LastIndexOf(".")).Trim();
                }
            }
            catch
            {
            }

            return className;
        }

        /// <summary>
        /// Gibt eine Liste von Assemblies zurck,
        /// die whrend der Ausnahme geladen wurden.
        /// </summary>
        private AssemblyPoint[] GetLoadedAssemblies()
        {
            ArrayList al = new ArrayList();

            try
            {
                Assembly[] assemblies = AppDomain.CurrentDomain.GetAssemblies();
                foreach (Assembly assembly in assemblies)
                {
                    AssemblyPoint ap = new AssemblyPoint();
                    try
                    {
                        try
                        {
                            ap.Name = assembly.GetName().Name;
                        }
                        catch (Exception ex)
                        {
                            ap.Name = "Exception: " + ex.Message;
                        }

                        try
                        {
                            ap.FullName = assembly.FullName;
                        }
                        catch (Exception ex)
                        {
                            ap.FullName = "Exception: " + ex.Message;
                        }

                        try
                        {
                            ap.Codebase = assembly.CodeBase;
                        }
                        catch (Exception ex)
                        {
                            ap.Codebase = "Exception: " + ex.Message;
                        }

                        bool hasLocation = false;
                        try
                        {
                            ap.Location = assembly.Location;
                            hasLocation = true;
                        }
                        catch (Exception ex)
                        {
                            ap.Location = "Exception: " + ex.Message;
                        }

                        try
                        {
                            ap.LoadFromcache = assembly.GlobalAssemblyCache;
                        }
                        catch
                        {
                            ap.LoadFromcache = false;
                        }

                        try
                        {
                            ap.Imageversion = assembly.ImageRuntimeVersion;
                        }
                        catch (Exception ex)
                        {
                            ap.Imageversion = "Exception: " + ex.Message;
                        }

                        FileAttributes? fa = null;
                        if (hasLocation == true)
                        {
                            fa = System.IO.File.GetAttributes(ap.Location);
                            ap.Attributes = new AttributesPoint();
                        }

                        // Hidden
                        if (fa.HasValue)
                            if ((fa & FileAttributes.Hidden) == FileAttributes.Hidden)
                                ap.Attributes.Hidden = true;
                            else
                                ap.Attributes.Hidden = false;

                        // Readonly
                        if (fa.HasValue)
                            if ((fa & FileAttributes.ReadOnly) == FileAttributes.ReadOnly)
                                ap.Attributes.ReadOnly = true;
                            else
                                ap.Attributes.ReadOnly = false;

                        // Archive
                        if (fa.HasValue)
                            if ((fa & FileAttributes.Archive) == FileAttributes.Archive)
                                ap.Attributes.Archive = true;
                            else
                                ap.Attributes.Archive = false;

                        // Compress
                        if (fa.HasValue)
                            if ((fa & FileAttributes.Compressed) == FileAttributes.Compressed)
                                ap.Attributes.Compress = true;
                            else
                                ap.Attributes.Compress = false;

                        // Encrypted
                        if (fa.HasValue)
                            if ((fa & FileAttributes.Encrypted) == FileAttributes.Encrypted)
                                ap.Attributes.Encrypted = true;
                            else
                                ap.Attributes.Encrypted = false;

                        // hinzufgen
                        al.Add(ap);
                    }
                    catch (Exception ex)
                    {
                        AssemblyPoint exPoint = new AssemblyPoint();
                        exPoint.Name = (char)0 + ap.Name + ": " + ex.Message;
                        al.Add(exPoint);
                    }
                }
            }
            catch (Exception ex)
            {
                // allgemein Ausnahme beim Abholen der Assemblies
                AssemblyPoint exPoint = new AssemblyPoint();
                exPoint.Name = (char)0 + ex.Message;
                al.Add(exPoint);
            }
            al.Sort();

            return (AssemblyPoint[])al.ToArray(typeof(AssemblyPoint));
        }

        /// <summary>
        /// Extrahiert den Klassennamen aus dem Stack
        /// Trace der aktuellen Exception.
        /// </summary>
        private string GetClassName(Exception ex)
        {
            string className = "";

            try
            {
                // Erstes StackFrame aus StackTrace ermitteln
                StackFrame frame = GetFirstStackFrame(ex);

                // Klassennamen aus StackFrame ermitteln
                if (frame != null)
                {
                    MethodBase method = frame.GetMethod();
                    className = method.DeclaringType.ToString();
                }
                else
                    className = GetClassNameFromStack(ex.StackTrace);
            }
            catch (SecurityException)
            {
                className = SR.DEFAULTPERMISSIONDENIEDMESSAGE;
            }
            catch
            {
                className = SR.DEFAULTGENERALDENIEDMESSAGE;
            }

            return className;
        }

        /// <summary>
        /// Ermittelt das erste StackFrame aus dem
        /// StackTrace der aktuellen Ausnahme.
        /// </summary>
        private StackFrame GetFirstStackFrame(System.Exception ex)
        {
            StackFrame frame;

            try
            {
                // StackTrace aus Exception erstellen
                StackTrace trace = new StackTrace(ex, true);

                // StackFrame aus StackTrace ermitteln
                frame = trace.GetFrame(0);
                return frame;
            }
            catch
            {
                return null;
            }
        }

        /// <summary>
        /// Gibt den Type der Ausnahme zurck.
        /// </summary>
        private string GetExceptionType(System.Exception ex)
        {
            try
            {
                return ex.GetType().FullName;
            }
            catch (SecurityException)
            {
                return SR.DEFAULTPERMISSIONDENIEDMESSAGE;
            }
            catch
            {
                return SR.DEFAULTGENERALDENIEDMESSAGE;
            }
        }

        /// <summary>
        /// Gibt die anzuzeigenden erweiterten Eigenschaften zurck.
        /// </summary>
        private DataPropertyCollection GetDataProperties(Exception ex)
        {
            DataPropertyCollection coll = new DataPropertyCollection();

            if (ex.Data != null &&
                ex.Data.Count > 0)
            {
                foreach (DictionaryEntry entry in ex.Data)
                {
                    if (entry.Key != null &&
                        string.IsNullOrEmpty(entry.Key.ToString()) == false &&
                        entry.Value != null &&
                        string.IsNullOrEmpty(entry.Value.ToString()) == false)
                    {
                        DataProperty dp = new DataProperty();
                        dp.Name = entry.Key.ToString();
                        dp.Value = entry.Value.ToString();
                        coll.Add(dp);
                    }
                }
            }
            return coll;
        }

        /// <summary>
        /// Gibt einen String zurck, der keine Html-Sonderzeichen mehr enthlt zurck.
        /// </summary>
        private string NormalizeToHtml(string text)
        {
            string tmp = System.Security.SecurityElement.Escape(text);
            tmp = tmp.Replace("&apos;", "&#39;"); // Hack
            return tmp;
        }

        /// <summary>
        /// Gibt den Produktnamen zurck
        /// </summary>
        /// <param name="assembly">Die Assembly.</param>
        private string GetAssemblyProduct(Assembly assembly)
        {
            object[] attribute = assembly.GetCustomAttributes(typeof(AssemblyProductAttribute), true);
            if (attribute != null && attribute[0] != null)
            {
                return (attribute[0] as AssemblyProductAttribute).Product;
            }
            else
            {
                return "-";
            }
        }

        /// <summary>
        /// Gibt die Version zurck
        /// </summary>
        /// <param name="assembly">Die Assembly.</param>
        private string GetAssemblyVersion(Assembly assembly)
        {
            return assembly.GetName().Version.ToString();
        }
    }
}