﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.IO;
using System.Runtime.InteropServices;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Controls.Primitives;
using System.Windows.Input;
using System.Windows.Interop;
using System.Windows.Media.Imaging;
using WpfCoreExceptionDialog.Dialogs.Resources;

namespace WpfCoreExceptionDialog.Dialogs
{
    public partial class ExceptionWindow : Window, INotifyPropertyChanged
    {
        private const string EXPORT_FILE_PREFIX = "ExcWin";
        private const int WM_NCLBUTTONDBLCLK = 0x00A3;
        private const int WM_SYSCOMMAND = 0x0112;
        private const int SC_MOVE = 0xF010;
        private const int SC_SIZE = 0xF000;
        private const int SC_MINIMIZE = 0xF020;
        private const int SC_MAXIMIZE = 0xF030;
        private const int SC_RESTORE = 0xF120;
        private const int MF_BYCOMMAND = 0x00000000;

        [DllImport("user32.dll")]
        private static extern IntPtr GetSystemMenu(IntPtr hWnd, bool bRevert);

        [DllImport("user32.dll")]
        private static extern bool DeleteMenu(IntPtr hMenu, uint uPosition, uint uFlags);

        [DllImport("user32.dll", SetLastError = true)]
        private static extern int GetWindowLong(IntPtr hWnd, int nIndex);

        [DllImport("user32.dll")]
        private static extern int SetWindowLong(IntPtr hWnd, int nIndex, int dwNewLong);

        public ExceptionWindow()
        {
            InitializeComponent();
        }

        public ExceptionWindow(Exception exception, ViewMode viewMode = ViewMode.Default, bool restartVisible = false) :
            this(null, exception, viewMode, restartVisible)
        {
        }

        public ExceptionWindow(Window owner, Exception exception, ViewMode viewMode = ViewMode.Default, bool restartVisible = false) : this()
        {
            this.Owner = owner;
            this.RestartVisible = restartVisible;
            this.ViewMode = viewMode;
            this.ExceptionParser = new ExceptionParser();
            this.ExceptionParser.Load(exception);
            this.CurrentPackage = this.ExceptionParser.CurrentPackage;
        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            this.Icon = (BitmapSource)this.FindResource("IRTitleIcons32x32");
            this.Left = -1000;
            this.DataContext = this;
            this.txtMessage.DataContext = this;
            lvStacktrace.DataContext = this;
            lvLoadedFiles.DataContext = this;
            lvProperties.DataContext = this;
            lvApplication.DataContext = this;
            InitCommands();
            SetAll();
            FillApplicationList();
            FillAssembliesList();
            FillExceptions();
        }

        private void Window_Closing(object sender, CancelEventArgs e)
        {
            //TODO: Restart
            DeleteEditorFiles();
        }

        private void Window_PreviewKeyDown(object sender, KeyEventArgs e)
        {
            switch (e.Key)
            {
                case Key.Escape:
                    this.Close();
                    e.Handled = true;
                    break;
            }
        }

        private void lvExceptions_SelectedItemChanged(object sender, RoutedPropertyChangedEventArgs<object> e)
        {
            if (lvExceptions.SelectedItem != null)
            {
                this.CurrentPoint = (ExceptionPoint)((TreeViewItem)lvExceptions.SelectedItem).Tag;
            }
        }

        private void FillMessage()
        {
            if (this.ExceptionParser != null)
            {
                if (this.CurrentPoint != null)
                {
                    this.CurrentMessage = this.CurrentPoint.Message;
                }
            }
        }

        private void FillExceptions()
        {
            if (this.ExceptionParser != null)
            {
                lvExceptions.Items.Clear();
                if (this.CurrentPackage != null)
                {
                    ExceptionPoint current = this.CurrentPackage.General.Exception;
                    TreeViewItem currentNode = GenerateTreeViewItem(
                        current.Type,
                        "IRTabGeneralProperty");
                    currentNode.Tag = current;
                    currentNode.IsExpanded = true;
                    lvExceptions.Items.Add(currentNode);

                    while (current.InnerException != null)
                    {
                        current = current.InnerException;
                        TreeViewItem thisNode = GenerateTreeViewItem(
                            current.Type,
                            "IRTabGeneralProperty");
                        thisNode.Tag = current;
                        thisNode.IsExpanded = true;
                        currentNode.Items.Add(thisNode);
                        currentNode = thisNode;
                    }
                }

                // select first
                if (lvExceptions.Items.Count > 0)
                {
                    ((TreeViewItem)lvExceptions.Items[0]).IsSelected = true;
                }
            }
        }

        private void FillPropertiesList()
        {
            if (this.ExceptionParser != null)
            {
                List<KeyValuePair<string, string>> current = new List<KeyValuePair<string, string>>();
                if (this.CurrentPoint != null)
                {
                    if (string.IsNullOrEmpty(this.CurrentPoint.Sourcefile) == false)
                    {
                        current.Add(
                            new KeyValuePair<string, string>(SR.GENERALTABSOURCEFILE, this.CurrentPoint.Sourcefile)
                        );
                    }

                    if (string.IsNullOrEmpty(this.CurrentPoint.Class) == false)
                    {
                        current.Add(
                            new KeyValuePair<string, string>(SR.GENERALTABCLASS, this.CurrentPoint.Class)
                        );
                    }

                    if (string.IsNullOrEmpty(this.CurrentPoint.Method) == false)
                    {
                        current.Add(
                            new KeyValuePair<string, string>(SR.GENERALTABMETHOD, this.CurrentPoint.Method)
                        );
                    }

                    if (this.CurrentPoint.Line != "0" ||
                        this.CurrentPoint.Column != "0")
                    {
                        current.Add(
                            new KeyValuePair<string, string>(SR.GENERALTABLINECOLUMN, this.CurrentPoint.Line + "/" + this.CurrentPoint.Column)
                        );
                    }

                    if (this.CurrentPoint.IlOffset != "0" ||
                        this.CurrentPoint.NativeOffset != "0")
                    {
                        current.Add(
                            new KeyValuePair<string, string>(SR.GENERALTABOFFSET, this.CurrentPoint.IlOffset + "/" + this.CurrentPoint.NativeOffset)
                        );
                    }

                    if (this.CurrentPoint.DataProperties != null &&
                        this.CurrentPoint.DataProperties.Count > 0)
                    {
                        foreach (DataProperty thisProperty in this.CurrentPoint.DataProperties)
                        {
                            current.Add(
                                new KeyValuePair<string, string>(thisProperty.Name, thisProperty.Value)
                            );
                        }
                    }
                }
                this.CurrentPropertiesList = current;
            }
        }

        private void FillStacktraceList()
        {
            if (this.ExceptionParser != null)
            {
                List<string> current = new List<string>();
                if (this.CurrentPoint != null &&
                    this.CurrentPoint.Stacks != null &&
                    this.CurrentPoint.Stacks.Count > 0)
                {
                    foreach (StackPoint thisPoint in this.CurrentPoint.Stacks)
                    {
                        current.Add(thisPoint.Text);
                    }
                }
                this.CurrentStacktraceList = current;
            }
        }

        private void FillAssembliesList()
        {
            if (this.ExceptionParser != null &&
                this.CurrentPackage != null &&
                this.CurrentPackage.Assemblies != null &&
                this.CurrentPackage.Assemblies.Count > 0)
            {
                lvLoadedFiles.Items.Clear();
                foreach (AssemblyPoint thisPoint in this.CurrentPackage.Assemblies)
                {
                    TreeViewItem assItem = GenerateTreeViewItem(
                        SR.ASSEMBLIESTABASSEMBLYTEXT + ": " + thisPoint.Name,
                        "IRTabAssembliesFile"
                     );
                    assItem.Items.Add(GenerateTreeViewItem(
                        SR.ASSEMBLIESTABFULLNAMETEXT + ": " + thisPoint.FullName,
                        "IRTabAssembliesName"
                        ));
                    assItem.Items.Add(GenerateTreeViewItem(
                        SR.ASSEMBLIESTABCODEBASETEXT + ": " + thisPoint.Codebase,
                        "IRTabAssembliesCodebase"
                        ));
                    assItem.Items.Add(GenerateTreeViewItem(
                        SR.ASSEMBLIESTABLOCATIONTEXT + ": " + thisPoint.Location,
                        "IRTabAssembliesLocation"
                        ));
                    assItem.Items.Add(GenerateTreeViewItem(
                        SR.ASSEMBLIESTABIMAGEVERSIONTEXT + ": " + thisPoint.Imageversion,
                        "IRTabAssembliesImage"
                        ));
                    assItem.Items.Add(GenerateTreeViewItem(
                        SR.ASSEMBLIESTABLOADFROMCACHETEXT + ": " + thisPoint.LoadFromcache.GetString(),
                        "IRTabAssembliesCache"
                        ));
                    if (thisPoint.Attributes != null)
                    {
                        TreeViewItem fileAttrItem = GenerateTreeViewItem(
                            SR.ASSEMBLIESTABFILEATTRIBUTETEXT,
                            "IRTabAssembliesAttribute"
                            );
                        fileAttrItem.Items.Add(GenerateTreeViewItem(
                            SR.ASSEMBLIESTABHIDDENTEXT + ": " + thisPoint.Attributes.Hidden.GetString(),
                            "IRTabAssemblies" + thisPoint.Attributes.Hidden.GetEnString()
                            ));
                        fileAttrItem.Items.Add(GenerateTreeViewItem(
                            SR.ASSEMBLIESTABREADONLYTEXT + ": " + thisPoint.Attributes.ReadOnly.GetString(),
                            "IRTabAssemblies" + thisPoint.Attributes.ReadOnly.GetEnString()
                            ));
                        fileAttrItem.Items.Add(GenerateTreeViewItem(
                            SR.ASSEMBLIESTABARCHIVETEXT + ": " + thisPoint.Attributes.Archive.GetString(),
                            "IRTabAssemblies" + thisPoint.Attributes.Archive.GetEnString()
                            ));
                        fileAttrItem.Items.Add(GenerateTreeViewItem(
                            SR.ASSEMBLIESTABCOMPRESSTEXT + ": " + thisPoint.Attributes.Compress.GetString(),
                            "IRTabAssemblies" + thisPoint.Attributes.Compress.GetEnString()
                            ));
                        fileAttrItem.Items.Add(GenerateTreeViewItem(
                            SR.ASSEMBLIESTABENCRYPTTEXT + ": " + thisPoint.Attributes.Encrypted.GetString(),
                            "IRTabAssemblies" + thisPoint.Attributes.Encrypted.GetEnString()
                            ));
                        assItem.Items.Add(fileAttrItem);
                    }
                    assItem.IsExpanded = true;
                    lvLoadedFiles.Items.Add(assItem);
                }
            }
        }

        private void FillApplicationList()
        {
            if (this.ExceptionParser != null &&
                this.CurrentPackage != null &&
                this.CurrentPackage.General != null)
            {
                List<KeyValuePair<string, string>> current = new List<KeyValuePair<string, string>>();

                current.Add(
                    new KeyValuePair<string, string>(SR.APPLICATIONTABTITLE, this.CurrentPackage.General.Title)
                );
                current.Add(
                    new KeyValuePair<string, string>(SR.APPLICATIONTABVERSION, this.CurrentPackage.General.Version)
                );
                current.Add(
                    new KeyValuePair<string, string>(SR.APPLICATIONTABSTARTUPPATH, this.CurrentPackage.General.StartupPath)
                );
                current.Add(
                    new KeyValuePair<string, string>(SR.APPLICATIONTABOS, this.CurrentPackage.General.Operatingsystem)
                );
                current.Add(
                    new KeyValuePair<string, string>(SR.APPLICATIONTABNETVERSION, this.CurrentPackage.General.NetVersion)
                );
                current.Add(
                    new KeyValuePair<string, string>(SR.APPLICATIONTABHOST, this.CurrentPackage.General.Host)
                );
                current.Add(
                    new KeyValuePair<string, string>(SR.APPLICATIONTABUSERNAME, this.CurrentPackage.General.User)
                );
                current.Add(
                    new KeyValuePair<string, string>(SR.APPLICATIONTABAPPDOMAIN, this.CurrentPackage.General.AppDomain)
                );
                current.Add(
                    new KeyValuePair<string, string>(SR.APPLICATIONTABASSEMBLY, this.CurrentPackage.General.Assembly)
                );
                current.Add(
                    new KeyValuePair<string, string>(SR.APPLICATIONTABTHREADNAMEUSERID, this.CurrentPackage.General.ThreadName + "/" + this.CurrentPackage.General.ThreadUser + "/" + this.CurrentPackage.General.ThreadId)
                );

                this.CurrentApplicationList = current;
            }
        }

        private void SetAll()
        {
            SetCloseButton();
            SetTitle();
            SetSize();
            SetTab();
            SetMoreFewerButtons();
            SetButtonsSameWidth();
        }

        private void SetSize()
        {
            if (this.ExceptionParser != null)
            {
                this.Left = -1000;
                if (this.ViewMode == ViewMode.Default)
                {
                    this.WindowState = WindowState.Normal;

                    if (System.Windows.SystemParameters.PrimaryScreenWidth < 500)
                        this.Width = System.Windows.SystemParameters.PrimaryScreenWidth;
                    else
                        this.Width = 500;

                    this.Height = rowHeader.ActualHeight +
                               rowSeperatorTop.ActualHeight +
                               rowMessage.ActualHeight +
                               rowSeperatorBottom.ActualHeight +
                               rowButtons.ActualHeight +
                               SystemParameters.CaptionHeight + 20;

                    this.CenterWindowOnScreen();
                }
                else
                {
                    this.ResizeMode = ResizeMode.CanResize;
                    this.WindowState = WindowState.Maximized;
                    this.ResizeMode = ResizeMode.NoResize;
                }
            }
        }

        private void SetTab()
        {
            if (this.ExceptionParser != null)
            {
                if (this.ViewMode == ViewMode.Default)
                {
                    tabMain.Visibility = Visibility.Collapsed;
                }
                else
                {
                    tabMain.Visibility = Visibility.Visible;
                }
            }
        }

        private void SetTitle()
        {
            if (this.ExceptionParser != null)
            {
                if (this.ViewMode == ViewMode.Default)
                {
                    lblSubtitle.Visibility = Visibility.Visible;
                }
                else
                {
                    lblSubtitle.Visibility = Visibility.Collapsed;
                }
            }
        }

        private void SetMoreFewerButtons()
        {
            if (this.ExceptionParser != null)
            {
                if (this.ViewMode == ViewMode.Default)
                {
                    btnMore.Visibility = Visibility.Visible;
                    btnFewer.Visibility = Visibility.Collapsed;
                }
                else
                {
                    btnMore.Visibility = Visibility.Collapsed;
                    btnFewer.Visibility = Visibility.Visible;
                }
            }
        }

        private void SetButtonsSameWidth()
        {
            if (this.ExceptionParser != null)
            {
                double w = 0;
                if (btnExport.ActualWidth > w)
                    w = btnExport.ActualWidth;

                if (btnFewer.ActualWidth > w)
                    w = btnFewer.ActualWidth;

                if (btnMore.ActualWidth > w)
                    w = btnMore.ActualWidth;

                if (btnOK.ActualWidth > w)
                    w = btnOK.ActualWidth;

                btnExport.Width = btnFewer.Width = btnMore.Width = btnOK.Width = w;
            }
        }

        private void SetCloseButton()
        {
            if (this.ExceptionParser != null)
            {
                IntPtr hwnd = new WindowInteropHelper(this).Handle;
                IntPtr hMenu = GetSystemMenu(hwnd, false);

                // MinMenü weg
                DeleteMenu(hMenu, SC_MINIMIZE, MF_BYCOMMAND);
                // MaxMenü weg
                DeleteMenu(hMenu, SC_MAXIMIZE, MF_BYCOMMAND);
                // SizeMenü weg
                DeleteMenu(hMenu, SC_SIZE, MF_BYCOMMAND);
                // RestoreMenü weg
                DeleteMenu(hMenu, SC_RESTORE, MF_BYCOMMAND);
            }
        }

        private void DeleteEditorFiles()
        {
            string[] files = Directory.GetFiles(Path.GetTempPath(), EXPORT_FILE_PREFIX + ".*.*");
            if (files != null && files.Length > 0)
            {
                foreach (string thisFiles in files)
                {
                    try
                    {
                        File.Delete(thisFiles);
                    }
                    catch { }
                }
            }
        }

        private TreeViewItem GenerateTreeViewItem(string text, string key, object tag = null)
        {
            TreeViewItem item = new TreeViewItem();
            item.Tag = tag;

            // create stack panel
            StackPanel stack = new StackPanel();
            stack.Orientation = Orientation.Horizontal;

            // create Image
            Image image = new Image();
            image.Source = (BitmapImage)this.FindResource(key);
            image.Width = 16;

            image.Height = 16;
            // Label
            Label lbl = new Label();
            lbl.Content = text;
            lbl.Padding = new Thickness(3);

            // Add into stack
            stack.Children.Add(image);
            stack.Children.Add(lbl);

            // assign stack to header
            item.Header = stack;
            return item;
        }

        #region --[ Bindungen ]--

        private Window _Owner;

        public new Window Owner
        {
            get
            {
                //if (_Owner == null)
                //{
                //    var helper = new WindowInteropHelper(this);
                //    helper.Owner = GetActiveWindow();
                //    return helper.Owner;
                //}
                //else
                //{
                return _Owner;
                //}
            }
            set
            {
                _Owner = value;
                RaisePropertyChanged(nameof(Owner));
            }
        }

        private ExceptionParser _ExceptionParser;

        private ExceptionParser ExceptionParser
        {
            get { return _ExceptionParser; }
            set
            {
                _ExceptionParser = value;
                RaisePropertyChanged(nameof(ExceptionParser));
            }
        }

        private string _CurrentMessage;

        public string CurrentMessage
        {
            get { return _CurrentMessage; }
            set
            {
                _CurrentMessage = value;
                RaisePropertyChanged(nameof(CurrentMessage));
            }
        }

        private List<string> _CurrentStacktraceList;

        public List<string> CurrentStacktraceList
        {
            get { return _CurrentStacktraceList; }
            set
            {
                _CurrentStacktraceList = value;
                RaisePropertyChanged(nameof(CurrentStacktraceList));
            }
        }

        private List<KeyValuePair<string, string>> _CurrentPropertiesList;

        public List<KeyValuePair<string, string>> CurrentPropertiesList
        {
            get { return _CurrentPropertiesList; }
            set
            {
                _CurrentPropertiesList = value;
                RaisePropertyChanged(nameof(CurrentPropertiesList));
            }
        }

        private List<KeyValuePair<string, string>> _CurrentApplicationList;

        public List<KeyValuePair<string, string>> CurrentApplicationList
        {
            get { return _CurrentApplicationList; }
            set
            {
                _CurrentApplicationList = value;
                RaisePropertyChanged(nameof(CurrentApplicationList));
            }
        }

        private ExceptionPoint _CurrentPoint;

        private ExceptionPoint CurrentPoint
        {
            get { return _CurrentPoint; }
            set
            {
                _CurrentPoint = value;
                RaisePropertyChanged(nameof(CurrentPoint));

                FillMessage();
                FillStacktraceList();
                FillPropertiesList();
            }
        }

        private ExceptionPackage _CurrentPackage;

        private ExceptionPackage CurrentPackage
        {
            get { return _CurrentPackage; }
            set
            {
                _CurrentPackage = value;
                RaisePropertyChanged(nameof(CurrentPackage));
            }
        }

        private ViewMode _ViewMode;

        private ViewMode ViewMode
        {
            get { return _ViewMode; }
            set
            {
                _ViewMode = value;
                RaisePropertyChanged(nameof(ViewMode));
                SetAll();
            }
        }

        private bool _RestartVisible;

        private bool RestartVisible
        {
            get { return _RestartVisible; }
            set
            {
                _RestartVisible = value;
                RaisePropertyChanged(nameof(RestartVisible));

                if (this.RestartVisible == true)
                    chkRestart.Visibility = Visibility.Visible;
                else
                    chkRestart.Visibility = Visibility.Collapsed;
            }
        }

        #endregion --[ Bindungen ]--

        #region --[ Commands ]--

        private void InitCommands()
        {
            OKCommand = new RelayCommand(OKCommandExecute, null);
            MoreCommand = new RelayCommand(MoreCommandExecute, null);
            FewerCommand = new RelayCommand(FewerCommandExecute, null);
            ClipboardCommand = new RelayCommand(ClipboardCommandExecute, null);
            EditorCommand = new RelayCommand(EditorCommandExecute, null);
            ExportCommand = new RelayCommand(ExportCommandExecute, null);
        }

        public RelayCommand ExportCommand { get; internal set; }

        private void ExportCommandExecute()
        {
            btnExport.ContextMenu.PlacementTarget = btnExport;
            btnExport.ContextMenu.Placement = PlacementMode.Bottom;
            btnExport.ContextMenu.IsOpen = true;
        }

        public RelayCommand ClipboardCommand { get; internal set; }

        private void ClipboardCommandExecute()
        {
            try
            {
                Clipboard.SetText(this.ExceptionParser.GetAsText());
            }
            catch (System.Exception ex)
            {
                MessageBox.Show(SR.DEFAULTPROCESSCANTSTARTMESSAGE.Replace("%1%", ex.Message),
                                SR.DEFAULTERRORTITLE,
                                MessageBoxButton.OK,
                                MessageBoxImage.Error);
            }
        }

        public RelayCommand EditorCommand { get; internal set; }

        private void EditorCommandExecute()
        {
            try
            {
                string fileName = Path.Combine(Path.GetTempPath(), EXPORT_FILE_PREFIX + "." + Guid.NewGuid().ToString("N") + ".txt");
                File.WriteAllText(fileName, this.ExceptionParser.GetAsText(), Encoding.UTF8);

                ProcessStartInfo pi = new ProcessStartInfo();
                pi.UseShellExecute = true;
                pi.FileName = fileName;
                Process.Start(pi);
            }
            catch (System.Exception ex)
            {
                MessageBox.Show(SR.DEFAULTPROCESSCANTSTARTMESSAGE.Replace("%1%", ex.Message),
                                SR.DEFAULTERRORTITLE,
                                MessageBoxButton.OK,
                                MessageBoxImage.Error);
            }
        }

        public RelayCommand OKCommand { get; internal set; }

        private void OKCommandExecute()
        {
            this.Close();
        }

        public RelayCommand MoreCommand { get; internal set; }

        private void MoreCommandExecute()
        {
            this.ViewMode = ViewMode.Extend;
        }

        public RelayCommand FewerCommand { get; internal set; }

        private void FewerCommandExecute()
        {
            this.ViewMode = ViewMode.Default;
        }

        #endregion --[ Commands ]--

        #region --[ INotifyPropertyChanged ]--

        public event PropertyChangedEventHandler PropertyChanged;

        private void RaisePropertyChanged(string property)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(property));
            }
        }

        #endregion --[ INotifyPropertyChanged ]--

        #region --[ WindowHook ]--

        private void Window_SourceInitialized(object sender, EventArgs e)
        {
            WindowInteropHelper helper = new WindowInteropHelper(this);
            HwndSource source = HwndSource.FromHwnd(helper.Handle);
            source.AddHook(WndProc);
        }

        private IntPtr WndProc(IntPtr hwnd, int msg, IntPtr wParam, IntPtr lParam, ref bool handled)
        {
            switch (msg)
            {
                case WM_SYSCOMMAND:
                    int command = wParam.ToInt32() & 0xfff0;
                    if (command == SC_MOVE &&
                        this.ViewMode == ViewMode.Extend)
                    {
                        // verschieben verhindern
                        handled = true;
                    }
                    else if (command == SC_SIZE)
                    {
                        // size verhindern
                        handled = true;
                    }
                    break;

                case WM_NCLBUTTONDBLCLK:
                    // Doppelclick verhindern
                    handled = true;
                    break;

                default:
                    break;
            }

            return IntPtr.Zero;
        }

        #endregion --[ WindowHook ]--
    }
}