using System;
using System.Reflection;

namespace WpfCoreExceptionDialog.Dialogs
{
    public class WeakAction
    {
        private Action _staticAction;

        protected MethodInfo Method
        {
            get;
            set;
        }

        public virtual string MethodName
        {
            get
            {
                if (this._staticAction != null)
                {
                    return this._staticAction.GetMethodInfo().Name;
                }
                return this.Method.Name;
            }
        }

        protected WeakReference ActionReference
        {
            get;
            set;
        }

        protected object LiveReference
        {
            get;
            set;
        }

        protected WeakReference Reference
        {
            get;
            set;
        }

        public bool IsStatic
        {
            get
            {
                return this._staticAction != null;
            }
        }

        public virtual bool IsAlive
        {
            get
            {
                if (this._staticAction == null && this.Reference == null && this.LiveReference == null)
                {
                    return false;
                }
                if (this._staticAction != null)
                {
                    return this.Reference == null || this.Reference.IsAlive;
                }
                return this.LiveReference != null || (this.Reference != null && this.Reference.IsAlive);
            }
        }

        public object Target
        {
            get
            {
                if (this.Reference == null)
                {
                    return null;
                }
                return this.Reference.Target;
            }
        }

        protected object ActionTarget
        {
            get
            {
                if (this.LiveReference != null)
                {
                    return this.LiveReference;
                }
                if (this.ActionReference == null)
                {
                    return null;
                }
                return this.ActionReference.Target;
            }
        }

        protected WeakAction()
        {
        }

        public WeakAction(Action action, bool keepTargetAlive = false) : this((action == null) ? null : action.Target, action, keepTargetAlive)
        {
        }

        public WeakAction(object target, Action action, bool keepTargetAlive = false)
        {
            if (action.GetMethodInfo().IsStatic)
            {
                this._staticAction = action;
                if (target != null)
                {
                    this.Reference = new WeakReference(target);
                }
                return;
            }
            this.Method = action.GetMethodInfo();
            this.ActionReference = new WeakReference(action.Target);
            this.LiveReference = (keepTargetAlive ? action.Target : null);
            this.Reference = new WeakReference(target);
        }

        public void Execute()
        {
            if (this._staticAction != null)
            {
                this._staticAction();
                return;
            }
            object actionTarget = this.ActionTarget;
            if (this.IsAlive && this.Method != null && (this.LiveReference != null || this.ActionReference != null) && actionTarget != null)
            {
                this.Method.Invoke(actionTarget, null);
                return;
            }
        }

        public void MarkForDeletion()
        {
            this.Reference = null;
            this.ActionReference = null;
            this.LiveReference = null;
            this.Method = null;
            this._staticAction = null;
        }
    }

    public class WeakAction<T> : WeakAction, IExecuteWithObject
    {
        private Action<T> _staticAction;

        public override string MethodName
        {
            get
            {
                if (this._staticAction != null)
                {
                    return this._staticAction.GetMethodInfo().Name;
                }
                return base.Method.Name;
            }
        }

        public override bool IsAlive
        {
            get
            {
                if (this._staticAction == null && base.Reference == null)
                {
                    return false;
                }
                if (this._staticAction != null)
                {
                    return base.Reference == null || base.Reference.IsAlive;
                }
                return base.Reference.IsAlive;
            }
        }

        public WeakAction(Action<T> action, bool keepTargetAlive = false) : this((action == null) ? null : action.Target, action, keepTargetAlive)
        {
        }

        public WeakAction(object target, Action<T> action, bool keepTargetAlive = false)
        {
            if (action.GetMethodInfo().IsStatic)
            {
                this._staticAction = action;
                if (target != null)
                {
                    base.Reference = new WeakReference(target);
                }
                return;
            }
            base.Method = action.GetMethodInfo();
            base.ActionReference = new WeakReference(action.Target);
            base.LiveReference = (keepTargetAlive ? action.Target : null);
            base.Reference = new WeakReference(target);
        }

        public new void Execute()
        {
            this.Execute(default(T));
        }

        public void Execute(T parameter)
        {
            if (this._staticAction != null)
            {
                this._staticAction(parameter);
                return;
            }
            object actionTarget = base.ActionTarget;
            if (this.IsAlive && base.Method != null && (base.LiveReference != null || base.ActionReference != null) && actionTarget != null)
            {
                base.Method.Invoke(actionTarget, new object[]
                {
                    parameter
                });
            }
        }

        public void ExecuteWithObject(object parameter)
        {
            T parameterCasted = (T)((object)parameter);
            this.Execute(parameterCasted);
        }

        public new void MarkForDeletion()
        {
            this._staticAction = null;
            base.MarkForDeletion();
        }
    }
}