using System;
using System.Reflection;

namespace WpfCoreExceptionDialog.Dialogs
{
    public class WeakFunc<TResult>
    {
        private Func<TResult> _staticFunc;

        protected MethodInfo Method
        {
            get;
            set;
        }

        public bool IsStatic
        {
            get
            {
                return this._staticFunc != null;
            }
        }

        public virtual string MethodName
        {
            get
            {
                if (this._staticFunc != null)
                {
                    return this._staticFunc.GetMethodInfo().Name;
                }
                return this.Method.Name;
            }
        }

        protected WeakReference FuncReference
        {
            get;
            set;
        }

        protected object LiveReference
        {
            get;
            set;
        }

        protected WeakReference Reference
        {
            get;
            set;
        }

        public virtual bool IsAlive
        {
            get
            {
                if (this._staticFunc == null && this.Reference == null && this.LiveReference == null)
                {
                    return false;
                }
                if (this._staticFunc != null)
                {
                    return this.Reference == null || this.Reference.IsAlive;
                }
                return this.LiveReference != null || (this.Reference != null && this.Reference.IsAlive);
            }
        }

        public object Target
        {
            get
            {
                if (this.Reference == null)
                {
                    return null;
                }
                return this.Reference.Target;
            }
        }

        protected object FuncTarget
        {
            get
            {
                if (this.LiveReference != null)
                {
                    return this.LiveReference;
                }
                if (this.FuncReference == null)
                {
                    return null;
                }
                return this.FuncReference.Target;
            }
        }

        protected WeakFunc()
        {
        }

        public WeakFunc(Func<TResult> func, bool keepTargetAlive = false) : this((func == null) ? null : func.Target, func, keepTargetAlive)
        {
        }

        public WeakFunc(object target, Func<TResult> func, bool keepTargetAlive = false)
        {
            if (func.GetMethodInfo().IsStatic)
            {
                this._staticFunc = func;
                if (target != null)
                {
                    this.Reference = new WeakReference(target);
                }
                return;
            }
            this.Method = func.GetMethodInfo();
            this.FuncReference = new WeakReference(func.Target);
            this.LiveReference = (keepTargetAlive ? func.Target : null);
            this.Reference = new WeakReference(target);
        }

        public TResult Execute()
        {
            if (this._staticFunc != null)
            {
                return this._staticFunc();
            }
            object funcTarget = this.FuncTarget;
            if (this.IsAlive && this.Method != null && (this.LiveReference != null || this.FuncReference != null) && funcTarget != null)
            {
                return (TResult)((object)this.Method.Invoke(funcTarget, null));
            }
            return default(TResult);
        }

        public void MarkForDeletion()
        {
            this.Reference = null;
            this.FuncReference = null;
            this.LiveReference = null;
            this.Method = null;
            this._staticFunc = null;
        }
    }

    public class WeakFunc<T, TResult> : WeakFunc<TResult>, IExecuteWithObjectAndResult
    {
        private Func<T, TResult> _staticFunc;

        public override string MethodName
        {
            get
            {
                if (this._staticFunc != null)
                {
                    return this._staticFunc.GetMethodInfo().Name;
                }
                return base.Method.Name;
            }
        }

        public override bool IsAlive
        {
            get
            {
                if (this._staticFunc == null && base.Reference == null)
                {
                    return false;
                }
                if (this._staticFunc != null)
                {
                    return base.Reference == null || base.Reference.IsAlive;
                }
                return base.Reference.IsAlive;
            }
        }

        public WeakFunc(Func<T, TResult> func, bool keepTargetAlive = false) : this((func == null) ? null : func.Target, func, keepTargetAlive)
        {
        }

        public WeakFunc(object target, Func<T, TResult> func, bool keepTargetAlive = false)
        {
            if (func.GetMethodInfo().IsStatic)
            {
                this._staticFunc = func;
                if (target != null)
                {
                    base.Reference = new WeakReference(target);
                }
                return;
            }
            base.Method = func.GetMethodInfo();
            base.FuncReference = new WeakReference(func.Target);
            base.LiveReference = (keepTargetAlive ? func.Target : null);
            base.Reference = new WeakReference(target);
        }

        public new TResult Execute()
        {
            return this.Execute(default(T));
        }

        public TResult Execute(T parameter)
        {
            if (this._staticFunc != null)
            {
                return this._staticFunc(parameter);
            }
            object funcTarget = base.FuncTarget;
            if (this.IsAlive && base.Method != null && (base.LiveReference != null || base.FuncReference != null) && funcTarget != null)
            {
                return (TResult)((object)base.Method.Invoke(funcTarget, new object[]
                {
                    parameter
                }));
            }
            return default(TResult);
        }

        public object ExecuteWithObject(object parameter)
        {
            T parameterCasted = (T)((object)parameter);
            return this.Execute(parameterCasted);
        }

        public new void MarkForDeletion()
        {
            this._staticFunc = null;
            base.MarkForDeletion();
        }
    }
}